/**
 * Module dependencies
 */
import $ from 'cafy';
import User from '../../../../models/user';
import Following from '../../../../models/following';
import queue from '../../../../queue';

/**
 * Follow a user
 *
 * @param {any} params
 * @param {any} user
 * @return {Promise<any>}
 */
module.exports = (params, user) => new Promise(async (res, rej) => {
	const follower = user;

	// Get 'userId' parameter
	const [userId, userIdErr] = $(params.userId).id().$;
	if (userIdErr) return rej('invalid userId param');

	// 自分自身
	if (user._id.equals(userId)) {
		return rej('followee is yourself');
	}

	// Get followee
	const followee = await User.findOne({
		_id: userId
	}, {
		fields: {
			data: false,
			'account.profile': false
		}
	});

	if (followee === null) {
		return rej('user not found');
	}

	// Check if already following
	const exist = await Following.findOne({
		followerId: follower._id,
		followeeId: followee._id,
		deletedAt: { $exists: false }
	});

	if (exist !== null) {
		return rej('already following');
	}

	// Create following
	const { _id } = await Following.insert({
		createdAt: new Date(),
		followerId: follower._id,
		followeeId: followee._id
	});

	queue.create('http', { type: 'follow', following: _id }).save();

	// Send response
	res();

});
